#ifndef __wasm_basics_string_h
#define __wasm_basics_string_h

/*
 * Include the real implementation, which is factored into a separate file so
 * that it can be reused by other libc string implementations.
 */
#include <__functions_memcpy.h>

#endif
