#ifndef __wasilibc___header_bits_signal_h
#define __wasilibc___header_bits_signal_h

#include <wasi/api.h>

#define SIGHUP    __WASI_SIGNAL_HUP
#define SIGINT    __WASI_SIGNAL_INT
#define SIGQUIT   __WASI_SIGNAL_QUIT
#define SIGILL    __WASI_SIGNAL_ILL
#define SIGTRAP   __WASI_SIGNAL_TRAP
#define SIGABRT   __WASI_SIGNAL_ABRT
#define SIGBUS    __WASI_SIGNAL_BUS
#define SIGFPE    __WASI_SIGNAL_FPE
#define SIGKILL   __WASI_SIGNAL_KILL
#define SIGUSR1   __WASI_SIGNAL_USR1
#define SIGSEGV   __WASI_SIGNAL_SEGV
#define SIGUSR2   __WASI_SIGNAL_USR2
#define SIGPIPE   __WASI_SIGNAL_PIPE
#define SIGALRM   __WASI_SIGNAL_ALRM
#define SIGTERM   __WASI_SIGNAL_TERM
#define SIGCHLD   __WASI_SIGNAL_CHLD
#define SIGCONT   __WASI_SIGNAL_CONT
#define SIGSTOP   __WASI_SIGNAL_STOP
#define SIGTSTP   __WASI_SIGNAL_TSTP
#define SIGTTIN   __WASI_SIGNAL_TTIN
#define SIGTTOU   __WASI_SIGNAL_TTOU
#define SIGURG    __WASI_SIGNAL_URG
#define SIGXCPU   __WASI_SIGNAL_XCPU
#define SIGXFSZ   __WASI_SIGNAL_XFSZ
#define SIGVTALRM __WASI_SIGNAL_VTALRM
#define SIGPROF   __WASI_SIGNAL_PROF
#define SIGWINCH  __WASI_SIGNAL_WINCH
#define SIGPOLL   __WASI_SIGNAL_POLL
#define SIGPWR    __WASI_SIGNAL_PWR
#define SIGSYS    __WASI_SIGNAL_SYS

#define SIGIOT    SIGABRT
#define SIGIO     SIGPOLL
#define SIGUNUSED SIGSYS

#endif
