#define CPU_HAS_FPU		0x0001
#define CPU_HAS_P2_FLUSH_BUG	0x0002
#define CPU_HAS_MMU_PAGE_ASSOC	0x0004
#define CPU_HAS_DSP		0x0008
#define CPU_HAS_PERF_COUNTER	0x0010
#define CPU_HAS_PTEA		0x0020
#define CPU_HAS_LLSC		0x0040
#define CPU_HAS_L2_CACHE	0x0080
#define CPU_HAS_OP32		0x0100
#define CPU_HAS_PTEAEX		0x0200
#define CPU_HAS_CAS_L		0x0400
